<?php
/**
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * $Revision: 1.7 $
 * $Author: tora60 $
 * $Date: 2009/11/26 00:34:55 $
 */

defined('mxMainFileLoaded') or die('access denied');

/**
 * Definition der jeweiligen Platzhalter und deren Ersetzungen
 */
function theme_define_placeholders()
{
    global $themesetting;
    $part[] = array('{DESIGN}', theme_get_design_id());
    $part[] = array("{LAYOUT_CLASS}", theme_get_layout_class());
    $part[] = array("{PAGE_TITLE}", $themesetting['title']);
    $part[] = array("{NAVBAR}", theme_get_navbars());
    $part[] = array("{FOOTMESSAGE}", theme_show_footmsg());
    $part[] = array("{DEBUGSERVICE}", theme_get_servicetext('debugservice'));
    $part[] = array("{SITESERVICE}", theme_get_servicetext('siteservice'));
    $part[] = array("{CURRENTYEAR}", date('Y'));

    $part[] = array("{DEBUGINFO}", ((function_exists('theme_get_debuginfo')) ? theme_get_debuginfo() : ''));

    return $part;
}

/**
 * Definition der einzelnen Blockbereiche
 */
function theme_define_blocks()
{
    global $themesetting;
    static $var;
    if (isset($var)) return $var;
    // linke Bloecke
    $name = 'block_left';
    $var[$name]['container'] = 'blocks_left_loop';
    $var[$name]['function'] = 'themesidebox';
    $var[$name]['position'] = 'l';
    /* einfache Moeglichkeit, die linken Bloecke bei bestimmten Seiten zu entfernen */
    if (theme_hideleftblocks() && MX_MODULE != 'admin') {
        unset($var[$name]);
    }
    // obere Center-Bloecke
    $name = 'block_center_top';
    $var[$name]['container'] = 'blocks_center_top_loop';
    $var[$name]['function'] = 'thememiddlebox';
    $var[$name]['position'] = 'c';
    $name = 'perex';
    $var[$name]['container'] = 'perex_loop';
    $var[$name]['function'] = 'theme_perex';
    $var[$name]['position'] = 'd';
    // untere Center-Bloecke
    $name = 'block_center_down';
    $var[$name]['container'] = 'blocks_center_down_loop';
    $var[$name]['function'] = 'thememiddlebox';
    $var[$name]['position'] = 'd';
    // rechte Bloecke
    $name = 'block_right_1';
    $var[$name]['container'] = 'blocks_right_loop';
    $var[$name]['function'] = 'themesidebox';
    $var[$name]['position'] = 'r';
    $name = 'block_right_2';
    $var[$name]['container'] = 'blocks_right_loop';
    $var[$name]['function'] = 'themesidebox';
    $var[$name]['position'] = 'r';
    return $var;
}

/**
 * Definition des Contentbereiches inkl. der open/close-Table Funktionen
 */
function theme_define_content()
{
    static $var;
    if (isset($var)) return $var;
    // die Funktion OpenTable()
    $var['opentabs']['OpenTable']['templatevar'] = 'opentable';
    $var['opentabs']['OpenTable']['innerreplace'] = '{OPENCLOSE_TABLE}'; // hier kann irgendwas stehen, es muss aber mit dem entsprechenden Text innerhalb der template-Datei uebereinstimmen...
    // die Funktion OpenTable2()
    $var['opentabs']['OpenTable2']['templatevar'] = 'opentable2';
    $var['opentabs']['OpenTable2']['innerreplace'] = '{OPENCLOSE_TABLE_2}';
    // die Funktion OpenTableAl()
    $var['opentabs']['OpenTableAl']['templatevar'] = 'opentableal';
    $var['opentabs']['OpenTableAl']['innerreplace'] = '{OPENCLOSE_TABLE_AL}';
    // die Funktion themeindex() / News Modul
    $var['themeindex'] = 'themeindex';
    // die Funktion themearticle() / News Modul
    $var['themearticle'] = 'themearticle';
    // der Name (output_container) darf nicht veraendert werden !!
    $var['output_container'] = 'script_output';
    // der Name (index_on_container) darf nicht veraendert werden !!
    $var['index_on_container'] = 'blocks_right_container';
    $var['index_on_block_container'] = 'blocks_right_loop';
    // der Name (more_header) darf nicht veraendert werden !!
    $var['add_header'] = 'more_header';

    return $var;
}

/**
 * ersetzen von eigenen Theme-Elementen, kann veraendert und ergaenzt werden
 * diese Teile werden gleich zu Beginn des scriptes, beim einlesen des templates, ersetzt
 * Vorsicht, wenn das Theme gecached werden soll!!!
 * Dann duerfen hier keine dynamischen Elemente eingesetzt werden.
 */
function theme_replace_start($template)
{
    $switch = new themeStyleSwitcher();
    theme_extract_optional_part($template, 'styleswitch_head', $switch->head());
    theme_extract_optional_part($template, 'styleswitch_view', $switch->view());

    /**
     * bestimmte Texte, vor allem Image-Pfade, die ersetzt werden sollen, definieren (suche/ersetze)
     */
    $part[] = array('"style/', '"' . MX_THEME_DIR . '/style/'); // die Stylesheets
    $part[] = array('"images/', '"' . MX_THEME_DIR . '/images/'); // normale images im theme
    $part[] = array('"design/', '"' . MX_THEME_DIR . '/design/'); // normale images im theme
    $part[] = array('url(design/', 'url(' . MX_THEME_DIR . '/design/');
    $part[] = array('url(images/', 'url(' . MX_THEME_DIR . '/images/');
    $part[] = array('"js/', '"' . MX_THEME_DIR . '/js/');
    // Suchformular entfernen, wenn Suche abgeschaltet ist
    if (!mxModuleAllowed('Search')) {
        theme_extract_optional_part($template, 'searchform', '<br />');
    }
    // Siteservicebereich entfernen, wenn Siteservice abgeschaltet oder leer ist
    if (!theme_get_servicetext('siteservice')) {
        theme_extract_optional_part($template, 'siteservice', '');
    }
    // Debugbereich entfernen, wenn Debugmodus abgeschaltet oder leer ist
    if (!theme_get_servicetext('debugservice')) {
        theme_extract_optional_part($template, 'debugservice', '');
    }
    // das Kopfmenue durch die tatsaechlichen Daten ersetzen
    theme_extract_optional_part($template, 'headmenue', '{NAVBAR}');

    return theme_replace_parts($template, $part);
}

/**
 * ersetzen von eigenen Theme-Elementen, kann veraendert und ergaenzt werden
 * diese Teile werden vor der Ausgabe des headers im Headbereich ersetzt
 */
function theme_replace_header($newheader)
{
    /* Das Farbschema ermitteln und an die sheme.php uebergeben */
    $newheader = str_replace('{DESIGN}', theme_get_design_id(), $newheader);

    /* doppelte Zeilenumbrueche und fuehrende Leerzeichen entfernen */
    $newheader = preg_replace('#\n+(\s+)?#', "\n", $newheader);

    return $newheader;
}

/**
 * ersetzen in jedem einzelnen Block
 */
function theme_replace_blocks($template, $block)
{
    global $themesetting;
    switch (true) {
        case $block['position'] === 'l' && strpos($block['blockfile'], 'block-Modules') === 0:
        case $block['position'] === 'l' && $block['blockfile'] === 'block-Menu.php':
            $template = '<div class="' . $themesetting['blocknav']['style'] . '">' . $template . '</div>';
            break;
            // case $block['position'] === 'r':
            // case $block['position'] === 'c':
            // case $block['position'] === 'd':
            // break;
    }

    /* Teil fuer alle Bloecke */
    if (empty($block['title']) || $block['title'] == 'NOTITLE') {
        $part[] = array('__BLOCK_ID__' , '__BLOCK_ID__ block-hide-caption');
    }
    $part[] = array('__BLOCK_ID__' , 'block-' . $block['position'] . '-' . $block['order']);

    if (isset($part)) {
        return theme_replace_parts($template, $part);
    }
    return $template;
}

/**
 * ersetzen von eigenen Theme-Elementen, kann veraendert und ergaenzt werden
 * diese Teile werden am Ende des scriptes, in der Funktion themefooter() ersetzt
 */
function theme_replace_end($template)
{
    global $themesetting;

    /* einfache Moeglichkeit, die linken Bloecke bei bestimmten Seiten zu entfernen */
    if (theme_hideleftblocks()) {
        theme_extract_part($template, 'blocks_left_container');
    }

    /* die haesslichen Rahmen um Checkboxen und Radio-Feldern entfernen, ueber style-sheet Klasse */
    $template = theme_fix_formtags($template);

    /* leere Tabellen Tags entfernen, wenn kein Inhalt von Modul angezeigt wird (blank_home) */
    $part['preg'][] = array('#<table[^>]*>\s*<tbody>\s*<tr>\s*<td>\s*</td>\s*</tr>\s*</tbody>\s*</table>\s*<br\s*/?>#si', '');

    /* Design festlegen */
    $design = base64_decode('PGEgaHJlZj0iaHR0cDovL3d3dy5udXZpb3RlbXBsYXRlcy5jb20iPkZyZWUgd2ViIHRlbXBsYXRlczwvYT4gcHJlc2VudGVkIGJ5IDxhIGhyZWY9Imh0dHA6Ly93d3cucWFydGluLmN6Ij5RYXJ0aW48L2E+');
    if (!strpos($template, $design)) {
        $part[] = array($template, $template . $design);
    }

    $template = theme_replace_parts($template, $part);
    return $template;
}

/**
 * Die verschiedenen Layout-Spalten definieren
 */
function theme_get_layout_class()
{
    switch (true) {
        case theme_hidesideblocks():
        case theme_hiderightblocks() && theme_hideleftblocks():
            // einspaltiges Layout
            return 'hideboth';

        case theme_hiderightblocks():
            // zweispaltiges Layout
            return 'hideright';

        case theme_hideleftblocks():
            // zweispaltiges Layout
            return 'hideleft';

        default:
            // dreispaltiges Layout > Standard
            return 'hidenone';
    }
}

/**
 * pruefen ob linke Bloecke ausgeblendet werden sollen
 */
function theme_hideleftblocks()
{
    global $themesetting;
    if ((isset($themesetting['hide-left']) && in_array(1, $themesetting['hide-left'])) || theme_hidesideblocks()) {
        return true;
    }
    $blocks = mxGetAllBlocks('l');
    if (!$blocks) {
        // fr nchsten Aufruf vereinfachen
        $themesetting['hide-left'][] = 1;
        return true;
    }
    return false;
}

/**
 * pruefen ob linke Bloecke ausgeblendet werden sollen
 */
function theme_hiderightblocks()
{
    global $themesetting;
    if (empty($GLOBALS['index']) || theme_hidesideblocks()) {
        return true;
    }
    $blocks = mxGetAllBlocks('r');
    if (!$blocks) {
        // fr nchsten Aufruf vereinfachen
        $GLOBALS['index'] = false;
        return true;
    }
    return false;
}

/**
 * pruefen ob linke Bloecke ausgeblendet werden sollen
 */
function theme_hidesideblocks()
{
    global $themesetting;
    return (isset($themesetting['hide-both']) && in_array(1, $themesetting['hide-both']));
}

/* die Navigationsbuttons im header, Quelle in settings.php */
function theme_get_navbars()
{
    global $themesetting;
    foreach ($themesetting['headnav'] as $title => $menue) {
        if (empty($current) && in_array(1, $menue['case'])) {
            $current = ' class="tray-active"';
        } else {
            $current = '';
        }
        $part[] = '<li' . $current . '><a href="' . $menue['link'] . '">' . $title . '</a></li>';
    }
    // mxDebugFuncVars($themesetting, $part);
    return implode("\n", $part);
}

/* Das Farb-Design ermitteln */
function theme_get_design_id($setcookie = true)
{
    /* ACHTUNG! diese Funktion wird auch in der style/style.css.php verwendet */
    global $themesetting;

    /* wenn Funktion bereits ausgefuehrt, Wert zurueckgeben und Ende */
    static $design = 0;
    if ($design) {
        return $design;
    }

    /* sicherstellen, dass designs ein Array ist */
    settype($themesetting['designs'], 'array');

    /* entweder spezielle Designs fuer bestimmte Seiten */
    if (isset($themesetting['pagedesign'])) {
        $design = array_shift($themesetting['designs']);
        foreach ($themesetting['pagedesign'] as $key => $cases) {
            if (in_array(true, $cases)) {
                $design = $key;
                break;
            }
        }
    }
    /* oder Styleswitcher fuer alle Seiten */
    else {
        /* URL-Parameter zur Design-Einstellung ist angegeben */
        if (isset($_GET['design'])) {
            $design = intval($_GET['design']);
        }

        /* Wenn statt dessen ein gueltiger Cookie vorhanden ist, diesen verwenden */
        if (!$design && isset($_COOKIE['style'])) {
            $design = intval($_COOKIE['style']);
        }

        /* falls angegebenes Design nicht vorhanden, das Erste verwenden */
        if (!$design || !in_array($design, $themesetting['designs'])) {
            $design = array_shift($themesetting['designs']);
        }
    }

    /* bei Bedarf den Cookie neu erstellen */
    if ($setcookie && (!isset($_COOKIE['style']) || $_COOKIE['style'] != $design)) {
        /* verfaellt in 365 Tagen */
        setcookie('style', $design, time() + 365 * 24 * 60 * 60);
    }

    /* gleich fuer Dateisystem formatieren (00) */
    $design = sprintf('%02d', $design);

    return $design;
}

/**
 * in $newheader ist der gesamte angepasste header enthalten
 */
function theme_header($newheader, &$siteservice, &$debugservice)
{
    global $theme_template, $themesetting;
    /* das Theme verwendet eigene Servicebereiche, deswegen hier die beiden Variablen killen */
    $siteservice = null;
    $debugservice = null;
    /* den body-Tag hinter head-Ende setzen >> nicht veraendern !! */
    $newheader .= "\n</head>\n\n" . $theme_template['body_tag'] . "\n\n";
    /* bei Bedarf den XHTML-Dokumenttyp ueberpruefen und Fehlermeldung ausgeben */
    if (MX_IS_ADMIN && !theme_check_xhtmldoctype()) {
        $newheader .= '<div class="warning"><h2 class="align-center">' . _FALSEDOCTYPE . '</h2></div>';
    }
    // ersetzen von eigenen Theme-Elementen, kann veraendert und ergaenzt werden
    // diese Teile werden vor der Ausgabe des headers im Headbereich ersetzt
    $newheader = theme_replace_header($newheader);

    return $newheader;
}

/**
 * parsen der Seiten-Bloecke
 */
function themesidebox($title, $content, $block = array(), $noecho = 0)
{
    global $theme_template, $themesetting;
    switch (true) {
        case empty($block):
        case empty($block['position']):
        case $block['position'] === 'l':
            $block['position'] = 'l';
            $out = $theme_template['block_left'];
            break;
        case $block['position'] === 'r' && in_array($block['bid'], $themesetting['block_right_alternate']):
            $out = $theme_template['block_right_2'];
            break;
        case $block['position'] === 'r':
            $out = $theme_template['block_right_1'];
            break;
    }
    $out = str_replace('{BLOCK_CONTENT}' , $content , $out);
    $out = str_replace('{BLOCK_TITLE}' , $title , $out);
    if ($noecho) {
        return $out;
    } else {
        echo $out;
    }
}

/**
 * Perex (der erste oberste Block) auslesen
 */
function theme_perex($title, $content, $block = array(), $noecho = 0)
{
    global $theme_template, $themesetting;

    if (!$themesetting['perex']) {
        return '';
    }

    static $perex = array();
    static $c_perex = 0;

    switch ($c_perex) {
        case 0:
            $blocks = mxGetAllBlocks('d');
            if (count($blocks) < 2) {
                $themesetting['perex'] = false;
                return;
            }
            $c_perex = 1;
            $perex['template'] = $theme_template['perex'];
            $perex['title'] = $title;
            $perex['content_1'] = $content;
            return '';
        case 1:
            $c_perex = 2;
            $out = str_replace('{PEREX_TITLE}' , $perex['title'] , $perex['template']);
            $out = str_replace('{PEREX_CONTENT1}' , $perex['content_1'] , $out);
            $out = str_replace('{PEREX_CONTENT2}' , $content , $out);
            if ($noecho) {
                return $out;
            }
            echo $out;
        default:
            return '';
    }
}

/**
 * parsen der Center-Bloecke
 */
function thememiddlebox($title, $content, $block = array(), $noecho = 0)
{
    global $theme_template, $themesetting;

    static $c_perex = 0;

    $out = '';
    switch (true) {
        case empty($block):
        case empty($block['position']):
        case $block['position'] === 'c':
            $block['position'] = 'c';
            $out = $theme_template['block_center_top'];
            break;

        case $block['position'] === 'd' && $themesetting['perex'] && $c_perex < 2 && $noecho === 'noecho':
            // 'noecho' wird nur in mx-themes zugefgt, so wird perex in Blockpreview u.A. nicht verwendet
            $c_perex++;
            return '';

        case $block['position'] === 'd':
            $out = $theme_template['block_center_down'];
            break;
    }

    $out = str_replace('{BLOCK_CONTENT}' , $content , $out);
    $out = str_replace('{BLOCK_TITLE}' , $title , $out);
    if ($noecho) {
        return $out;
    } else {
        echo $out;
    }
}

/**
 * News Modul Artikelliste (index.php)
 * $x bedeutet: nicht verwendet, nur zur nuke-Modulkompatibilitaet
 */
function themeindex($x, $x, $x, $x, $x, $x, $x, $x, $x, $x, $x, $x, $story = array())
{
    global $theme_template;
    pmxHeader::add_style(MX_THEME_DIR . '/style/news.css');
    // nur eine Spalte zulassen :-)
    $GLOBALS['storyhome_cols'] = 1;
    $cattitle = (empty($story['cattitle'])) ? '' : ' <span class="cat">(' . $story['allmorelink']['categorie'] . ')</span>';
    $story['linkedtitle'] = '<a href="modules.php?name=News&amp;file=article&amp;sid=' . $story['sid'] . '">' . strip_tags($story['title']) . '</a>' . $cattitle;
    $authorinfo = (empty($story['informant']) || $story['informant'] == $GLOBALS['anonymous']) ? theme_adminname($story) : " " . $story['allmorelink']['informantlink'] . $story['informant'] . "</a>";
    $story['topicimage'] = mxCreateImage($GLOBALS['tipath'] . $story['topicimage'], $story['topictext'], 0, 'align="left" class="story-topicimage"');
    $story['content'] = $story['hometext'];
    if (empty($story['allmorelink']['bodycount'])) {
        $story['readmore'] = '';
    } else {
        $title = ($story['allmorelink']['bodycount']) ? ' title="' . $story['allmorelink']['bodycount'] . ' ' . _BYTESMORE . '"' : '';
        $story['readmore'] = '<a href="modules.php?name=News&amp;file=article&amp;sid=' . $story['sid'] . '"' . $title . '>' . _HREADMORE . '</a>';
    }
    $story['infoline'] = _NEWSSUBMITED . " " . $authorinfo . " " . $story['allmorelink']['datetime'];
    // die oben definierten Variablen in dem passenden templateteil ersetzen
    $artvars = theme_define_content();
    echo theme_replace_vars($theme_template[$artvars['themeindex']], $story);
    return;
}

/**
 * News Modul Artikelansicht (article.php)
 * $x bedeutet: nicht verwendet, nur zur nuke-Modulkompatibilitaet
 */
function themearticle($x, $x, $x, $x, $x, $x, $x, $x, $x, $story = array())
{
    global $theme_template;
    pmxHeader::add_style(MX_THEME_DIR . '/style/news.css');
    $authorinfo = (empty($story['informant']) || $story['informant'] == $GLOBALS['anonymous']) ? theme_adminname($story) : " " . $story['allmorelink']['informantlink'] . $story['informant'] . "</a>";
    $story['infoline'] = _NEWSSUBMITED . " " . $authorinfo . " " . $story['allmorelink']['datetime'];
    $story['topicimage'] = mxCreateImage($GLOBALS['tipath'] . $story['topicimage'], $story['topictext'], 0, 'align="left" class="story-topicimage"');
    $story['title'] = strip_tags($story['title']);
    $story['content'] = $story['hometext'] . "<br />\n" . $story['bodytext'];
    $story['notes'] = (empty($story['notes'])) ? '' : '<p class="notes">' . _NOTE . '</p><p>' . $story['notes'] . '</p>';
    // die oben definierten Variablen in dem passenden templateteil ersetzen
    echo theme_replace_vars($theme_template['themearticle'], $story);
    return;
}

/**
 * themeStyleSwitcher
 *
 * @package
 * @author tora60
 * @copyright Copyright (c) 2009
 * @version $Id: functions.php,v 1.7 2009/11/26 00:34:55 tora60 Exp $
 * @access public
 */
class themeStyleSwitcher {
    private $_head;
    private $_view;

    /**
     * themeStyleSwitcher::__construct()
     */
    public function __construct()
    {
        $set = $GLOBALS['themesetting'];
        $current = theme_get_design_id();
        $head = false;

        /* normaler Styleswitch zum auswaehlen */
        if ($this->checkcookie() && !isset($set['pagedesign']) && isset($set['designs']) && count($set['designs'] > 1)) {
            /* sicherstellen, dass designs ein Array ist */
            settype($set['designs'], 'array');
            /* url aufsplitten */
            $query = $_SERVER['QUERY_STRING'];
            if (isset($_GET['design'])) {
                $query = preg_replace('#[&?]?design=[0-9]*#', '', $query);
            }
            $to = basename($_SERVER['PHP_SELF']);
            // index.php ist auch php_self=modules.php, deswegen hier index.php verwenden, falls $name leer ist
            if ($to == 'modules.php' && empty($_GET['name'])) {
                $to = './';
            }
            if ($query) {
                $to .= '?' . str_replace('&', '&amp;', $query) . '&amp;design=';
            } else {
                $to .= '?design=';
            }

            foreach ($set['designs'] as $design) {
                $design = sprintf('%02d', $design);
                if ($current != $design) {
                    $head[] = '<link rel="alternate stylesheet" type="text/css" href="design/' . $design . '/scheme.css" title="' . $design . '" />';
                }
                $view[] = '<li><a style="background-image: url(design/' . $design . '/header.gif)" href="' . $to . '' . $design . '" rel="' . $design . '">&nbsp;</a></li>';
            }
            if ($head) {
                if (method_exists('pmxHeader', 'add_jquery')) {
                    pmxHeader::add_jquery(); // ab pragmaMx 1.12
                } else {
                    pmxHeader::add_script(PMX_JAVASCRIPT_PATH . 'jquery/jquery.js');
                }
                pmxHeader::add_script(MX_THEME_DIR . '/js/styleswitch.js');
                pmxHeader::add_script_code('var cookiepath="' . PMX_BASE_PATH . '";');

                $this->_head = "\n" . implode("\n", $head);

                $this->_view = '
                <ul id="styleswitch" class="noprint" title="' . _THEME_DESIGN . '">'
                 . implode("\n", $view) . '
                </ul><br class="clear" />
                ';
            }
        }
        /* falls kein Cookie vorhanden, design manuell im header eintragen */
        else if (isset($set['pagedesign']) && (!isset($_COOKIE['style']) || $_COOKIE['style'] != $current)) {
            pmxHeader::add_style(MX_THEME_DIR . '/design/' . $current . '/scheme.css');
        }
    }

    /**
     * themeStyleSwitcher::checkcookie()
     * check, ob cookies aktiviert
     *
     * @return
     */
    private function checkcookie()
    {
        switch (true) {
            case isset($_COOKIE['style']):
                return true;
            case empty($_SERVER['HTTP_REFERER']):
                return false;
            default:
                $info = pathinfo($_SERVER['HTTP_REFERER']);
                if (strpos(PMX_HOME_URL, $info['dirname']) === 0) {
                    return false;
                }
                return true;
        }
    }

    /**
     * themeStyleSwitcher::head()
     * Variablen fuer den <head> Bereich
     *
     * @return
     */
    public function head()
    {
        return $this->_head;
    }

    /**
     * themeStyleSwitcher::view()
     * Variablen fuer den <body> Bereich
     *
     * @return
     */
    public function view()
    {
        return $this->_view;
    }
}

?>